/**
 * Copyright (c) 2015-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.objectweb.petals.tools.rmi.server.remote.implementations;

import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.logging.Logger;

import javax.jbi.JBIException;
import javax.jbi.management.MBeanNames;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.management.MBeanServer;
import javax.naming.InitialContext;
import javax.xml.namespace.QName;

import org.objectweb.petals.tools.rmi.remote.RemoteComponentContext;
import org.objectweb.petals.tools.rmi.remote.RemoteComponentContextClient;
import org.objectweb.petals.tools.rmi.remote.RemoteDeliveryChannel;
import org.objectweb.petals.tools.rmi.server.remote.implementations.RemoteComponentContextClientFactoryImpl.SharedComponentData;
import org.w3c.dom.Document;
import org.w3c.dom.DocumentFragment;

import com.ebmwebsourcing.easycommons.uuid.QualifiedUUIDGenerator;

/**
 * The RMI implementation of component client context.
 * 
 * @author Christophe DENEUX - Linagora
 */
public class RemoteComponentContextClientImpl extends UnicastRemoteObject implements
        RemoteComponentContextClient {

    private static final QualifiedUUIDGenerator GENERATOR = new QualifiedUUIDGenerator("PetalsSE-RMIClient");

    private static final long serialVersionUID = 8492024345313032709L;

    /**
     * The remote component context associated to the current Petals SE RMI.
     */
    private final RemoteComponentContext remoteComponentContext;

    /**
     * A remote delivery channel associated to the current TMI client
     */
    private final RemoteDeliveryChannel remoteDeliveryChannel;

    private final SharedComponentData sharedData;

    private final String rmiClientId;

    /**
     * the constructor.
     * 
     * @param c
     *            the component context
     * @throws RemoteException
     *             impossible to realize a rmi access
     */
    public RemoteComponentContextClientImpl(final SharedComponentData sharedData) throws JBIException, RemoteException {
        this.sharedData = sharedData;
        this.rmiClientId = GENERATOR.getNewID();
        
        this.sharedData.allReceivedMessages.putIfAbsent(this.rmiClientId, new LinkedBlockingQueue<MessageExchange>());

        this.remoteComponentContext = new RemoteComponentContextImpl(this.sharedData, this.rmiClientId);

        this.remoteDeliveryChannel = new RemoteDeliveryChannelImpl(this.sharedData, this.rmiClientId);
    }

    @Override
    public ServiceEndpoint activateEndpoint(final QName interfaceName, final QName serviceName,
            final String endpointName) throws JBIException, RemoteException {
        return this.remoteComponentContext.activateEndpoint(interfaceName, serviceName, endpointName);
    }

    @Override
    public void deactivateEndpoint(final ServiceEndpoint endpoint) throws JBIException, RemoteException {
        this.remoteComponentContext.deactivateEndpoint(endpoint);
    }

    @Override
    public void deregisterExternalEndpoint(final ServiceEndpoint externalEndpoint) throws JBIException, RemoteException {
        this.remoteComponentContext.deregisterExternalEndpoint(externalEndpoint);
    }

    @Override
    public String getComponentName() throws RemoteException {
        return this.remoteComponentContext.getComponentName();
    }

    @Override
    public ServiceEndpoint getEndpoint(final QName service, final String name) throws RemoteException {
        return this.remoteComponentContext.getEndpoint(service, name);
    }

    @Override
    public Document getEndpointDescriptor(final ServiceEndpoint endpoint) throws JBIException, RemoteException {
        return this.remoteComponentContext.getEndpointDescriptor(endpoint);
    }

    @Override
    public ServiceEndpoint[] getEndpoints(final QName interfaceName) throws RemoteException {
        return this.remoteComponentContext.getEndpoints(interfaceName);
    }

    @Override
    public ServiceEndpoint[] getEndpointsForService(final QName serviceName) throws RemoteException {
        return this.remoteComponentContext.getEndpointsForService(serviceName);
    }

    @Override
    public ServiceEndpoint[] getExternalEndpoints(final QName interfaceName) throws RemoteException {
        return this.remoteComponentContext.getExternalEndpoints(interfaceName);
    }

    @Override
    public ServiceEndpoint[] getExternalEndpointsForService(final QName serviceName) throws RemoteException {
        return this.remoteComponentContext.getExternalEndpointsForService(serviceName);
    }

    @Override
    public String getInstallRoot() throws RemoteException {
        return this.remoteComponentContext.getInstallRoot();
    }

    @Override
    public Logger getLogger(final String suffix, final String resourceBundleName) throws JBIException, RemoteException {
        // A logger is not serializable
        throw new NoSuchMethodError("getLogger: Not implemented.");
    }

    @Override
    public void logConsumesExtFlowStepBegin(final String flowInstanceId, final String flowStepId,
            final String flowInterfaceName, final String flowServiceName, final String flowEndpointName,
            final String flowOperationName) throws RemoteException {
        this.remoteComponentContext.logConsumesExtFlowStepBegin(flowInstanceId, flowStepId, flowInterfaceName,
                flowServiceName, flowEndpointName, flowOperationName);
    }

    @Override
    public void logConsumesExtFlowStepEnd(final String flowInstanceId, final String flowStepId) throws RemoteException {
        this.remoteComponentContext.logConsumesExtFlowStepEnd(flowInstanceId, flowStepId);
    }

    @Override
    public void logConsumesExtFlowStepFailure(final String flowInstanceId, final String flowStepId, final String msg)
            throws RemoteException {
        this.remoteComponentContext.logConsumesExtFlowStepFailure(flowInstanceId, flowStepId, msg);
    }

    @Override
    public void log(final java.util.logging.Level level, final String msg, final Object param1) throws RemoteException {
        this.remoteComponentContext.log(level, msg, param1);
    }

    @Override
    public MBeanNames getMBeanNames() throws RemoteException {
        return this.remoteComponentContext.getMBeanNames();
    }

    @Override
    public MBeanServer getMBeanServer() throws RemoteException {
        return this.remoteComponentContext.getMBeanServer();
    }

    @Override
    public InitialContext getNamingContext() throws RemoteException {
        return this.remoteComponentContext.getNamingContext();
    }

    @Override
    public Object getTransactionManager() throws RemoteException {
        return this.remoteComponentContext.getTransactionManager();
    }

    @Override
    public String getWorkspaceRoot() throws RemoteException {
        return this.remoteComponentContext.getWorkspaceRoot();
    }

    @Override
    public void registerExternalEndpoint(final ServiceEndpoint externalEndpoint) throws JBIException, RemoteException {
        this.remoteComponentContext.registerExternalEndpoint(externalEndpoint);
    }

    @Override
    public ServiceEndpoint resolveEndpointReference(final DocumentFragment epr) throws RemoteException {
        return this.remoteComponentContext.resolveEndpointReference(epr);
    }

    @Override
    public RemoteDeliveryChannel getDeliveryChannel() throws RemoteException {
        return this.remoteDeliveryChannel;
    }

    @Override
    public RemoteComponentContext getComponentContext() throws RemoteException {
        return this.remoteComponentContext;
    }
}
